@@ -1,7 +1,7 @@
 #include <config.h>
 
 #include "rcsid.h"
-RCSID ("$Id: pwdcheck.c,v 1.2 2003/04/22 10:59:22 kloczek Exp $")
+RCSID ("$Id: pwdcheck.c,v 1.4 2004/11/02 18:46:30 kloczek Exp $")
 #include "prototypes.h"
 #include "defines.h"
 #include <pwd.h>
@@ -32,9 +32,9 @@ passwd_check (const char *user, const char *passwd, const char *progname)
 		goto bailout;
 
 	retcode = pam_acct_mgmt (pamh, 0);
-	if (retcode == PAM_NEW_AUTHTOK_REQD) {
+	if (retcode == PAM_NEW_AUTHTOK_REQD)
 		retcode = pam_chauthtok (pamh, PAM_CHANGE_EXPIRED_AUTHTOK);
-	} else if (retcode)
+	if (retcode)
 		goto bailout;
 
 	if (pam_setcred (pamh, 0))
