@@ -30,7 +30,7 @@
 #include <config.h>
 
 #include "rcsid.h"
-RCSID (PKG_VER "$Id: useradd.c,v 1.32 2002/01/10 13:01:28 kloczek Exp $")
+RCSID (PKG_VER "$Id: useradd.c,v 1.50 2004/11/09 02:45:43 kloczek Exp $")
 #include "prototypes.h"
 #include "defines.h"
 #include "chkname.h"
@@ -55,6 +55,7 @@ RCSID (PKG_VER "$Id: useradd.c,v 1.32 2002/01/10 13:01:28 kloczek Exp $")
 #include "lastlog_.h"
 #endif
 #include "faillog.h"
+#include "nscd.h"
 #ifndef SKEL_DIR
 #define SKEL_DIR "/etc/skel"
 #endif
@@ -76,6 +77,7 @@ static const char *def_gname = "other";
 static const char *def_home = "/home";
 static const char *def_shell = "";
 static const char *def_template = SKEL_DIR;
+static const char *def_create_mail_spool = "no";
 
 #ifdef SHADOWPWD
 static long def_inactive = -1;
@@ -93,6 +95,7 @@ static gid_t user_gid;
 static const char *user_comment = "";
 static const char *user_home = "";
 static const char *user_shell = "";
+static const char *create_mail_spool = "";
 
 #ifdef	SHADOWPWD
 static long user_expire = -1;
@@ -108,26 +111,20 @@ static int do_grp_update = 0;	/* group files need to be updated */
 static char *Prog;
 
 static int
- bflg = 0,	/* new default root of home directory */
- cflg = 0,	/* comment (GECOS) field for new account */
- dflg = 0,	/* home directory for new account */
- Dflg = 0,	/* set/show new user default values */
- eflg = 0,	/* days since 1970-01-01 when account is locked */
- fflg = 0,	/* days until account with expired password is locked */
- gflg = 0,	/* primary group ID for new account */
- Gflg = 0,	/* secondary group set for new account */
- kflg = 0,	/* specify a directory to fill new user directory */
- mflg = 0,	/* create user's home directory if it doesn't exist */
- nflg = 0,	/* create a group having the same name as the user */
- oflg = 0,	/* permit non-unique user ID to be specified with -u */
- sflg = 0,	/* shell program for new account */
- uflg = 0;	/* specify user ID for new account */
-
-#ifdef AUTH_METHODS
-static int Aflg = 0;		/* specify authentication method for user */
-static char user_auth[1024];
-static char *auth_arg;
-#endif
+ bflg = 0,			/* new default root of home directory */
+ cflg = 0,			/* comment (GECOS) field for new account */
+ dflg = 0,			/* home directory for new account */
+ Dflg = 0,			/* set/show new user default values */
+ eflg = 0,			/* days since 1970-01-01 when account is locked */
+ fflg = 0,			/* days until account with expired password is locked */
+ gflg = 0,			/* primary group ID for new account */
+ Gflg = 0,			/* secondary group set for new account */
+ kflg = 0,			/* specify a directory to fill new user directory */
+ mflg = 0,			/* create user's home directory if it doesn't exist */
+ nflg = 0,			/* create a group having the same name as the user */
+ oflg = 0,			/* permit non-unique user ID to be specified with -u */
+ sflg = 0,			/* shell program for new account */
+ uflg = 0;			/* specify user ID for new account */
 
 extern char *optarg;
 extern int optind;
@@ -138,6 +135,7 @@ extern int pw_dbm_mode;
 #ifdef	SHADOWPWD
 extern int sp_dbm_mode;
 #endif
+
 extern int gr_dbm_mode;
 
 #ifdef	SHADOWGRP
@@ -178,29 +176,22 @@ static int sg_dbm_added;
  */
 #define E_SUCCESS	0	/* success */
 #define E_PW_UPDATE	1	/* can't update password file */
-#define E_USAGE		2	/* bad command syntax */
+#define E_USAGE		2	/* invalid command syntax */
 #define E_BAD_ARG	3	/* invalid argument to option */
 #define E_UID_IN_USE	4	/* uid already in use (and no -o) */
 #define E_NOTFOUND	6	/* specified group doesn't exist */
 #define E_NAME_IN_USE	9	/* username already in use */
 #define E_GRP_UPDATE	10	/* can't update group file */
 #define E_HOMEDIR	12	/* can't create home directory */
+#define	E_MAIL_SPOOL	13	/* can't create mail spool */
 
-#ifdef SVR4
-#define DGROUP	"defgroup="
-#define HOME	"defparent="
-#define SHELL	"defshell="
-#define INACT	"definact="
-#define EXPIRE	"defexpire="
-#define SKEL	"defskel="
-#else
-#define DGROUP	"GROUP="
-#define HOME	"HOME="
-#define SHELL	"SHELL="
-#define INACT	"INACTIVE="
-#define EXPIRE	"EXPIRE="
-#define SKEL	"SKEL="
-#endif
+#define DGROUP			"GROUP="
+#define HOME			"HOME="
+#define SHELL			"SHELL="
+#define INACT			"INACTIVE="
+#define EXPIRE			"EXPIRE="
+#define SKEL			"SKEL="
+#define CREATE_MAIL_SPOOL	"CREATE_MAIL_SPOOL="
 
 /* local function prototypes */
 static void fail_exit (int);
@@ -221,10 +212,6 @@ static void new_spent (struct spwd *);
 static void grp_update (void);
 static void find_new_uid (void);
 
-#ifdef AUTH_METHODS
-static void convert_auth (char *, const char *);
-static int valid_auth (const char *);
-#endif
 static void process_flags (int argc, char **argv);
 static void close_files (void);
 static void open_files (void);
@@ -232,6 +219,7 @@ static void faillog_reset (uid_t);
 static void lastlog_reset (uid_t);
 static void usr_update (void);
 static void create_home (void);
+static void create_mail (void);
 
 /*
  * fail_exit - undo as much as possible
@@ -354,7 +342,8 @@ static void get_defaults (void)
 		 */
 
 		if (MATCH (buf, DGROUP)) {
-			unsigned int val = (unsigned int) strtoul (cp, &ep, 10);
+			unsigned int val =
+			    (unsigned int) strtoul (cp, &ep, 10);
 
 			if (*cp != '\0' && *ep == '\0') {	/* valid number */
 				def_group = val;
@@ -378,7 +367,6 @@ static void get_defaults (void)
 		/*
 		 * Default HOME filesystem
 		 */
-
 		else if (MATCH (buf, HOME)) {
 			def_home = xstrdup (cp);
 		}
@@ -386,15 +374,14 @@ static void get_defaults (void)
 		/*
 		 * Default Login Shell command
 		 */
-
 		else if (MATCH (buf, SHELL)) {
 			def_shell = xstrdup (cp);
 		}
 #ifdef SHADOWPWD
+
 		/*
 		 * Default Password Inactive value
 		 */
-
 		else if (MATCH (buf, INACT)) {
 			long val = strtol (cp, &ep, 10);
 
@@ -407,7 +394,6 @@ static void get_defaults (void)
 		/*
 		 * Default account expiration date
 		 */
-
 		else if (MATCH (buf, EXPIRE)) {
 			def_expire = xstrdup (cp);
 		}
@@ -416,13 +402,22 @@ static void get_defaults (void)
 		/*
 		 * Default Skeleton information
 		 */
-
 		else if (MATCH (buf, SKEL)) {
 			if (*cp == '\0')
 				cp = SKEL_DIR;	/* XXX warning: const */
 
 			def_template = xstrdup (cp);
 		}
+
+		/*
+		 * Create by default user mail spoll or not ?
+		 */
+		else if (MATCH (buf, CREATE_MAIL_SPOOL)) {
+			if (*cp == '\0')
+				cp = CREATE_MAIL_SPOOL;	/* XXX warning: const */
+
+			def_create_mail_spool = xstrdup (cp);
+		}
 	}
 }
 
@@ -436,26 +431,15 @@ static void get_defaults (void)
 
 static void show_defaults (void)
 {
-#ifdef SVR4
-	printf (_("group=%s,%u  basedir=%s  skel=%s\n"),
-		def_gname, (unsigned int) def_group, def_home,
-		def_template);
-
-	printf (_("shell=%s  "), def_shell);
+	printf ("GROUP=%u\n", (unsigned int) def_group);
+	printf ("HOME=%s\n", def_home);
 #ifdef SHADOWPWD
-	printf (_("inactive=%ld  expire=%s"), def_inactive, def_expire);
+	printf ("INACTIVE=%ld\n", def_inactive);
+	printf ("EXPIRE=%s\n", def_expire);
 #endif
-	printf ("\n");
-#else				/* !SVR4 */
-	printf (_("GROUP=%u\n"), (unsigned int) def_group);
-	printf (_("HOME=%s\n"), def_home);
-#ifdef SHADOWPWD
-	printf (_("INACTIVE=%ld\n"), def_inactive);
-	printf (_("EXPIRE=%s\n"), def_expire);
-#endif
-	printf (_("SHELL=%s\n"), def_shell);
-	printf (_("SKEL=%s\n"), def_template);
-#endif				/* !SVR4 */
+	printf ("SHELL=%s\n", def_shell);
+	printf ("SKEL=%s\n", def_template);
+	printf ("CREATE_MAIL_SPOOL=%s\n", def_create_mail_spool);
 }
 
 /*
@@ -480,20 +464,13 @@ static int set_defaults (void)
 	int out_expire = 0;
 	int out_shell = 0;
 	int out_skel = 0;
-
-#ifdef	SVR4
-	int out_gname = 0;
-#endif
+	int out_create_mail_spool = 0;
 
 	/*
 	 * Create a temporary file to copy the new output to.
 	 */
 
-#ifdef HAVE_MKSTEMP
 	if ((ofd = mkstemp (new_file)) == -1) {
-#else
-	if ((ofd = mktemp (new_file)) == -1) {
-#endif
 		fprintf (stderr,
 			 _("%s: cannot create new defaults file\n"), Prog);
 		return -1;
@@ -524,14 +501,7 @@ static int set_defaults (void)
 			fprintf (ofp, DGROUP "%u\n",
 				 (unsigned int) def_group);
 			out_group++;
-		}
-#ifdef	SVR4
-		else if (!out_gname && MATCH (buf, "defgname=")) {
-			fprintf (ofp, "defgname=%s\n", def_gname);
-			out_gname++;
-		}
-#endif
-		else if (!out_home && MATCH (buf, HOME)) {
+		} else if (!out_home && MATCH (buf, HOME)) {
 			fprintf (ofp, HOME "%s\n", def_home);
 			out_home++;
 #ifdef	SHADOWPWD
@@ -541,14 +511,18 @@ static int set_defaults (void)
 		} else if (!out_expire && MATCH (buf, EXPIRE)) {
 			fprintf (ofp, EXPIRE "%s\n", def_expire);
 			out_expire++;
-		}
 #endif
-		else if (!out_shell && MATCH (buf, SHELL)) {
+		} else if (!out_shell && MATCH (buf, SHELL)) {
 			fprintf (ofp, SHELL "%s\n", def_shell);
 			out_shell++;
 		} else if (!out_skel && MATCH (buf, SKEL)) {
 			fprintf (ofp, SKEL "%s\n", def_template);
 			out_skel++;
+		} else if (!out_create_mail_spool
+			   && MATCH (buf, CREATE_MAIL_SPOOL)) {
+			fprintf (ofp, CREATE_MAIL_SPOOL "%s\n",
+				 def_create_mail_spool);
+			out_create_mail_spool++;
 		} else
 			fprintf (ofp, "%s\n", buf);
 	}
@@ -576,6 +550,10 @@ static int set_defaults (void)
 	if (!out_skel)
 		fprintf (ofp, SKEL "%s\n", def_template);
 
+	if (!out_create_mail_spool)
+		fprintf (ofp, CREATE_MAIL_SPOOL "%s\n",
+			 def_create_mail_spool);
+
 	/*
 	 * Flush and close the file. Check for errors to make certain
 	 * the new file is intact.
@@ -612,12 +590,17 @@ static int set_defaults (void)
 	}
 #ifdef SHADOWPWD
 	SYSLOG ((LOG_INFO,
-		 "defaults: group=%u, home=%s, inactive=%ld, expire=%s",
-		 (unsigned int) def_group, def_home, def_inactive,
-		 def_expire));
+		 "useradd defaults: GROUP=%u, HOME=%s, SHELL=%s, INACTIVE=%ld, "
+		 "EXPIRE=%s, SKEL=%s, CREATE_MAIL_SPOOL=%s",
+		 (unsigned int) def_group, def_home, def_shell,
+		 def_inactive, def_expire, def_template,
+		 def_create_mail_spool));
 #else
-	SYSLOG ((LOG_INFO, "defaults: group=%u, home=%s",
-		 (unsigned int) def_group, def_home));
+	SYSLOG ((LOG_INFO,
+		 "useradd defaults: GROUP=%u, HOME=%s, SHELL=%s, "
+		 "SKEL=%s, CREATE_MAIL_SPOOL=%s",
+		 (unsigned int) def_group, def_home, def_shell,
+		 def_template, def_create_mail_spool));
 #endif
 	return 0;
 }
@@ -735,25 +718,18 @@ static void usage (void)
 {
 	fprintf (stderr,
 		 _
-		 ("usage: %s\t[-u uid [-o]] [-g group] [-G group,...] \n"),
-		 Prog);
+		 ("Usage: useradd [-u uid [-o]] [-g group] [-G group,...] \n"));
 	fprintf (stderr,
 		 _
-		 ("\t\t[-d home] [-s shell] [-c comment] [-m [-k template]]\n"));
-	fprintf (stderr, "\t\t");
+		 ("               [-d home] [-s shell] [-c comment] [-m [-k template]]\n"));
 #ifdef SHADOWPWD
-	fprintf (stderr, _("[-f inactive] [-e expire ] "));
+	fprintf (stderr, _("               [-f inactive] [-e expire]\n"));
 #endif
-#ifdef AUTH_METHODS
-	fprintf (stderr, _("[-A program] "));
-#endif
-	fprintf (stderr, _("[-p passwd] name\n"));
-
+	fprintf (stderr, _("               [-p passwd] name\n"));
 	fprintf (stderr,
-		 _("       %s\t-D [-g group] [-b base] [-s shell]\n"),
-		 Prog);
+		 _("       useradd -D [-g group] [-b base] [-s shell]\n"));
 #ifdef SHADOWPWD
-	fprintf (stderr, _("\t\t[-f inactive] [-e expire ]\n"));
+	fprintf (stderr, _("               [-f inactive] [-e expire]\n"));
 #endif
 
 	exit (E_USAGE);
@@ -777,18 +753,9 @@ static void new_pwent (struct passwd *pwent)
 #endif
 		pwent->pw_passwd = (char *) user_pass;
 
-#ifdef ATT_AGE
-	pwent->pw_age = (char *) "";
-#endif
 	pwent->pw_uid = user_id;
 	pwent->pw_gid = user_gid;
 	pwent->pw_gecos = (char *) user_comment;
-#ifdef	ATT_COMMENT
-	pwent->pw_comment = (char *) "";
-#endif
-#ifdef BSD_QUOTA
-	pwent->pw_quota = 0;
-#endif
 	pwent->pw_dir = (char *) user_home;
 	pwent->pw_shell = (char *) user_shell;
 }
@@ -1073,77 +1040,6 @@ static void find_new_uid (void)
 	}
 }
 
-#ifdef AUTH_METHODS
-/*
- * convert_auth - convert the argument list to a authentication list
- */
-
-static void convert_auth (char *auths, const char *list)
-{
-	char *cp, *end;
-	char buf[257];
-
-	/*
-	 * Copy each method. DEFAULT is replaced by an encrypted string
-	 * if one can be found in the current authentication list.
-	 */
-
-	strcpy (buf, list);
-	auths[0] = '\0';
-	for (cp = buf; cp; cp = end) {
-		if (auths[0])
-			strcat (auths, ";");
-
-		if ((end = strchr (cp, ',')))
-			*end++ = '\0';
-
-		if (strcmp (cp, "DEFAULT") == 0) {
-			strcat (auths, user_pass);
-		} else {
-			strcat (auths, "@");
-			strcat (auths, cp);
-		}
-	}
-}
-
-/*
- * valid_auth - check authentication list for validity
- */
-
-static int valid_auth (const char *methods)
-{
-	char *cp, *end;
-	char buf[257];
-	int default_cnt = 0;
-
-	/*
-	 * Cursory checks, length and illegal characters
-	 */
-
-	if ((int) strlen (methods) > 256)
-		return 0;
-
-	if (!VALID (methods))
-		return 0;
-
-	/*
-	 * Pick each method apart and check it.
-	 */
-
-	strcpy (buf, methods);
-	for (cp = buf; cp; cp = end) {
-		if ((end = strchr (cp, ',')))
-			*end++ = '\0';
-
-		if (strcmp (cp, "DEFAULT") == 0) {
-			if (default_cnt++ > 0)
-				return 0;
-		}
-	}
-	return 1;
-}
-#endif				/* AUTH_METHODS */
-
 /*
  * process_flags - perform command line argument setting
  *
@@ -1167,18 +1063,6 @@ static void process_flags (int argc, char **argv)
 	while ((arg = getopt (argc, argv, FLAGS)) != EOF) {
 #undef FLAGS
 		switch (arg) {
-#ifdef AUTH_METHODS
-		case 'A':
-			if (!valid_auth (optarg)) {
-				fprintf (stderr,
-					 _("%s: invalid field `%s'\n"),
-					 Prog, optarg);
-				exit (E_BAD_ARG);
-			}
-			auth_arg = optarg;
-			Aflg++;
-			break;
-#endif
 		case 'b':
 			if (!Dflg)
 				usage ();
@@ -1534,13 +1418,6 @@ static void usr_update (void)
 	if (!oflg)
 		find_new_uid ();
 
-#ifdef AUTH_METHODS
-	if (Aflg) {
-		convert_auth (user_auth, auth_arg);
-		user_pass = user_auth;
-	}
-#endif
-
 	/*
 	 * Fill in the password structure with any new fields, making
 	 * copies of strings.
@@ -1556,32 +1433,10 @@ static void usr_update (void)
 	 * happens so we know what we were trying to accomplish.
 	 */
 
-#ifdef AUTH_METHODS
-	SYSLOG ((LOG_INFO,
-		 "new user: name=%s, uid=%u, gid=%u, home=%s, shell=%s, auth=%s",
-		 user_name, (unsigned int) user_id, (unsigned int) user_gid, 
-		 user_home, user_shell, Aflg ? auth_arg : "DEFAULT"));
-#else
 	SYSLOG ((LOG_INFO,
 		 "new user: name=%s, uid=%u, gid=%u, home=%s, shell=%s",
-		 user_name, (unsigned int) user_id, (unsigned int) user_gid, 
-		 user_home, user_shell));
-#endif
-
-#ifdef AUTH_METHODS
-	/*
-	 * Attempt to add the new user to any authentication programs
-	 * which have been requested. Since this is more likely to fail
-	 * than the update of the password file, we do this first.
-	 */
-
-	if (Aflg && pw_auth (user_auth, pwent.pw_name, PW_ADD, (char *) 0)) {
-		fprintf (stderr,
-			 _("%s: error adding authentication method\n"),
-			 Prog);
-		fail_exit (E_PW_UPDATE);	/* XXX */
-	}
-#endif				/* AUTH_METHODS */
+		 user_name, (unsigned int) user_id,
+		 (unsigned int) user_gid, user_home, user_shell));
 
 	/*
 	 * Initialize faillog and lastlog entries for this UID in case
@@ -1684,15 +1539,63 @@ static void create_home (void)
 			fail_exit (E_HOMEDIR);
 		}
 		chown (user_home, user_id, user_gid);
-#if 1
-		chmod (user_home, 0777 & ~getdef_num ("UMASK", 077));
-#else
-		chmod (user_home, 0755);
-#endif
+		chmod (user_home, 0777 & ~getdef_num ("UMASK", 022));
 		home_added++;
 	}
 }
 
+/*
+ * create_mail - create the user's mail spool
+ *
+ *	create_mail() creates the user's mail spool if it does not already
+ *	exist. It will be created mode 660 owned by the user and group
+ *	'mail'
+ */
+static void create_mail (void)
+{
+	char *ms;
+	int fd;
+	struct group *mail;
+	gid_t mail_gid;
+	mode_t mode;
+
+	if (strcasecmp (create_mail_spool, "yes") == 0) {
+		mail = getgrnam ("mail");
+		if (mail == NULL) {
+			fprintf (stderr,
+				 _
+				 ("No group named \"mail\" exists, creating mail spool with mode 0600.\n"));
+			mode = 0600;
+			mail_gid = user_gid;
+		} else {
+			mode = 0660;
+			mail_gid = mail->gr_gid;
+		}
+
+		ms = malloc (strlen (user_name) + 11);
+		if (ms != NULL) {
+			sprintf (ms, "/var/mail/%s", user_name);
+			if (access (ms, R_OK) != 0) {
+				fd = open (ms,
+					   O_CREAT | O_EXCL | O_WRONLY |
+					   O_TRUNC);
+				if (fd != -1) {
+					fchown (fd, user_id, mail_gid);
+					fchmod (fd, mode);
+					close (fd);
+				}
+			} else {
+				fprintf (stderr,
+					 _
+					 ("Can't create mail spool for user %s.\n"),
+					 user_name);
+				fail_exit (E_MAIL_SPOOL);
+			}
+		}
+		free (ms);
+	}
+}
+
 #ifdef USE_PAM
 static struct pam_conv conv = {
 	misc_conv,
@@ -1711,18 +1614,33 @@ int main (int argc, char **argv)
 	struct passwd *pampw;
 	int retval;
 #endif
+
 	/*
 	 * Get my name so that I can use it to report errors.
 	 */
 
-	sys_ngroups = sysconf (_SC_NGROUPS_MAX);
-	user_groups = malloc ((1 + sys_ngroups) * sizeof (char *));
 	Prog = Basename (argv[0]);
 
 	setlocale (LC_ALL, "");
 	bindtextdomain (PACKAGE, LOCALEDIR);
 	textdomain (PACKAGE);
 
+	OPENLOG("useradd");
+
+	sys_ngroups = sysconf(_SC_NGROUPS_MAX);
+	user_groups = malloc((1 + sys_ngroups) * sizeof(char *));
+
+#ifdef SHADOWPWD
+	is_shadow_pwd = spw_file_present();
+#endif
+#ifdef SHADOWGRP
+	is_shadow_grp = sgr_file_present();
+#endif
+
+	get_defaults();
+
+	process_flags(argc, argv);
+
 #ifdef USE_PAM
 	retval = PAM_SUCCESS;
 
@@ -1755,17 +1673,10 @@ int main (int argc, char **argv)
 			 Prog);
 		exit (1);
 	}
+
+	OPENLOG("useradd");
 #endif				/* USE_PAM */
 
-	OPENLOG (Prog);
-
-#ifdef SHADOWPWD
-	is_shadow_pwd = spw_file_present ();
-#endif
-#ifdef SHADOWGRP
-	is_shadow_grp = sgr_file_present ();
-#endif
-
 	/*
 	 * The open routines for the NDBM files don't use read-write
 	 * as the mode, so we have to clue them in.
@@ -1781,9 +1692,6 @@ int main (int argc, char **argv)
 	sg_dbm_mode = O_RDWR;
 #endif
 #endif
-	get_defaults ();
-
-	process_flags (argc, argv);
 
 	/*
 	 * See if we are messing with the defaults file, or creating
@@ -1825,8 +1733,13 @@ int main (int argc, char **argv)
 	}
 
 	/*
-	 * Do the hard stuff - open the files, create the user entries,
-	 * create the home directory, then close and update the files.
+	 * Do the hard stuff:
+	 * - open the files,
+	 * - create the user entries,
+	 * - create the home directory,
+	 * - create user mail spoll,
+	 * - flush nscd caches for passwd and group services,
+	 * - then close and update the files.
 	 */
 
 	open_files ();
@@ -1855,6 +1768,11 @@ int main (int argc, char **argv)
 			 Prog);
 	}
 
+	create_mail ();
+
+	nscd_flush_cache ("passwd");
+	nscd_flush_cache ("group");
+
 	close_files ();
 
 #ifdef USE_PAM
@@ -1875,4 +1793,5 @@ int main (int argc, char **argv)
 #endif				/* USE_PAM */
 
 	exit (E_SUCCESS);
- /*NOTREACHED*/}
+	/* NOT REACHED */
+}
